$TITLE Ramsey Model: Primal NLP Formulation

SET     T /1*50/
SET     TFIRST(T), TLAST(T);
TFIRST(T) = YES$(ORD(T) EQ 1);
TLAST(T)  = YES$(ORD(T) EQ CARD(T)); 

SCALAR          G       Growth rate             /0.02/
                R       Interest rate           /0.05/
                K0      Capital-output ratio    /3.00/
                DELTA   Depreciation rate       /0.07/

                KVS     Base year capital value share
                C0      Base year consumption;
                
C0 = 1 - (DELTA + G) * K0;

PARAMETER       QREF(T) Reference quantity path
                PREF(T) Reference price path;

QREF(T) = (1+G)**(ORD(T)-1);
PREF(T) = (1/(1+R))**(ORD(T)-1);

KVS = K0 * (DELTA + R);

PARAMETER ALPHA;
ALPHA(T) = ((1+G)/(1+R))**(ORD(T)-1);
ALPHA(TLAST) = ALPHA(TLAST) / (1-(1+G)/(1+R));

POSITIVE VARIABLES  
     K(T)       Capital stock
     C(T)       Consumption
     I(T)       Investment;

VARIABLES
     UTILITY    Utility function;

EQUATIONS 
     CC(T)      Capacity constraint,
     KK(T)      Capital balance,
     TC(T)      Terminal condition (provides for post-terminal growth),
     UTIL       Discounted log of consumption: objective function;


CC(T)..         (K(T)/K0)**KVS * QREF(T)**(1-KVS) =E= C(T) + I(T);

KK(T+1)..       (1-DELTA)*K(T) + I(T) =G= K(T+1);

TC(TLAST)..     I(TLAST) =G= (G+DELTA)*K(TLAST);

UTIL..          UTILITY =E= -SUM(T, ALPHA(T)*LOG(C(T)));

MODEL PRIMAL /CC, KK, TC, UTIL/;

C.L(T)          = C0;
K.LO(T)         = 1E-4;
C.LO(T)         = 1E-4;
I.LO(T)         = 1E-4;
K.FX(TFIRST)    = K0;

SOLVE PRIMAL MINIMIZING UTILITY USING NLP;

K.FX(TFIRST)    = K0*0.80;
SOLVE PRIMAL MINIMIZING UTILITY USING NLP;

PARAMETER       SOLUTION        Solution values;

SOLUTION(T,"PK") = KK.M(T)/CC.M(T);
SOLUTION(TFIRST,"PK") = -K.M(TFIRST)/CC.M(TFIRST);
SOLUTION(T,"C") = C.L(T);
SOLUTION(T,"I") = I.L(T);
SOLUTION(T,"K") = K.L(T);

DISPLAY SOLUTION;



$TITLE Ramsey Model: MCP based on KKT conditions for primal model

SET     T /1*50/
SET     TFIRST(T), TLAST(T);
TFIRST(T) = YES$(ORD(T) EQ 1);
TLAST(T)  = YES$(ORD(T) EQ CARD(T)); 

SCALAR          G       Growth rate             /0.02/
                R       Interest rate           /0.05/
                K0      Capital-output ratio    /3.00/
                DELTA   Depreciation rate       /0.07/
                Kstock  Capital index           /1.00/

                KVS     Base year capital value share
                I0      Base year investment
                C0      Base year consumption
                L0      Base year labor input
                U0      Baseline expenditure;
                

I0 = (DELTA + G) * K0;
C0 = 1 - I0;
L0 = 1 - K0 * (DELTA + R);

PARAMETER       QREF(T) Reference quantity path
                PREF(T) Reference price path;

QREF(T) = (1+G)**(ORD(T)-1);
PREF(T) = (1/(1+R))**(ORD(T)-1);

KVS = K0 * (DELTA + R);

PARAMETER ALPHA;
ALPHA(T) = ((1+G)/(1+R))**(ORD(T)-1);
ALPHA(TLAST) = ALPHA(TLAST) / (1-(1+G)/(1+R));


POSITIVE VARIABLES
     P(T)       Price of output,
     PK(T)      Price of capital,
     RK(T)      Price of rental capital,
     PTC(T)     Price of capital in terminal period
     K(T)       Capital stock
     C(T)       Consumption
     I(T)       Investment;

EQUATIONS
     CC(T)      Capacity constraint,
     KK(T)      Capital balance,
     TC(T)      Terminal condition (provides for post-terminal growth),
     OPT_C(T)   First order optimality condition for C,
     OPT_K(T)   First order optimality condition for K,
     OPT_I(T)   First order optimality condition for I;


CC(T)..         (K(T)/K0)**KVS * QREF(T)**(1-KVS) =E= C(T) + I(T);
KK(T)..         (1-DELTA)*K(T-1) + (K0*KSTOCK)$TFIRST(T) + I(T-1) =G= K(T);
TC(TLAST)..     I(TLAST) =G= (G+DELTA)*K(TLAST);

OPT_C(T)..     C(T)*P(T) =E= C0 * ALPHA(T);

OPT_K(T)..     PK(T) + (PTC(T)*(G+DELTA))$TLAST(T) =E= 
                P(T)*KVS*(K(T)/K0)**KVS * QREF(T)**(1-KVS)/K(T) + PK(T+1)*(1-DELTA);

OPT_I(T)..     P(T) =E= PK(T+1) + PTC(T)$TLAST(T);

MODEL KKT / OPT_C.C, OPT_K.K, OPT_I.I, CC.P, KK.PK, TC.PTC /;

*    Initialize price levels

C.L(T)  = C0 * QREF(T);
I.L(T)  = I0 * QREF(T);
K.L(T)  = K0 * QREF(T);

P.L(T)  = PREF(T);
PK.L(T) = (1+R) * PREF(T);
PTC.L(T) = PREF(T);

PK.LO(T)  = 1E-6;
PK.UP(T)  = +INF;

SOLVE KKT USING MCP;


KSTOCK = 0.8;

SOLVE KKT USING MCP;

PARAMETER       SOLUTION        Solution values;

SOLUTION(T,"PK") = PK.L(T) / P.L(T);
SOLUTION(T,"C") = C.L(T);
SOLUTION(T,"I") = I.L(T);
SOLUTION(T,"K") = K.L(T);

DISPLAY SOLUTION;


$TITLE Ramsey Model: MCP based on the MPSGE formulation 

$if not setglobal termcnd $setglobal termcnd MCP

SCALAR NLPTERM /0/, MCPTERM/0/;
%termcnd%term = 1;

SET     T /1*50/
SET     TFIRST(T), TLAST(T);
TFIRST(T) = YES$(ORD(T) EQ 1);
TLAST(T)  = YES$(ORD(T) EQ CARD(T)); 

SCALAR          G       Growth rate             /0.02/
                R       Interest rate           /0.05/
                K0      Capital-output ratio    /3.00/
                DELTA   Depreciation rate       /0.07/
                Kstock  Capital index           /1.00/

                KVS     Base year capital value share
                L0      Base year labor input;
                
L0 = 1 - K0 * (DELTA + R);

PARAMETER       QREF(T) Reference quantity path
                PREF(T) Reference price path;

QREF(T) = (1+G)**(ORD(T)-1);
PREF(T) = (1/(1+R))**(ORD(T)-1);

KVS = K0 * (DELTA + R);

PARAMETER ALPHA;
ALPHA(T) = ((1+G)/(1+R))**(ORD(T)-1);
ALPHA(TLAST)$NLPTERM = ALPHA(TLAST) / (1-(1+G)/(1+R));

ALIAS (T,TT);
ALPHA(T) = ALPHA(T) / SUM(TT, ALPHA(TT));

POSITIVE VARIABLES
        Y(T)    Output
        I(T)    Investment
        K(T)    Capital stock
        P(T)    Output price
        RK(T)   Return to capital
        U       Utility
        PU      Unit expenditure function
        PK(T)   Capital price
        PL(T)   Wage rate
        PKT     Terminal capital
        RA      Representative agent
        TK      Post-terminal capital stock;

EQUATIONS
     PR_Y(T)    Zero profit condition for output,
     PR_C(T)    Zero profit condition for consumption,
     PR_K(T)    Zero profit condition for capital,
     PR_I(T)    Zero profit condition for investment,
     PR_U       Zero profit condition for utility,

     M_P(T)     Market clearing for output, 
     M_PK(T)    Market clearing for capital,
     M_RK(T)    Market clearing for rental capital,
     M_PL(T)     Market clearing for labor,
     M_PU       Market clearing for utility,
     M_PKT      Market clearing for terminal investment,
     
     I_RA       Income balance for representative agent,

     TERMK      Terminal constraint for capital stock;

*    Zero profit conditions

PR_Y(T).. (RK(T)/(R+DELTA))**KVS * PL(T)**(1-KVS) =E= P(T);

PR_U..    PROD(T, (P(T)/PREF(T))**ALPHA(T) ) =E= PU;

PR_K(T).. PK(T) + ( (PKT*(G+DELTA))$TLAST(T) )$NLPTERM =E= 
                RK(T) + (1-DELTA)*(PK(T+1) + (PKT$TLAST(T))$MCPTERM);

PR_I(T).. P(T) =E= PK(T+1) + PKT$TLAST(T);

*    Market clearing conditions

M_P(T)..  Y(T) =E= ALPHA(T) * PU * U / P(T) + I(T);

M_PU..    U*PU =E= RA;

M_PK(T).. K(T) =E= (1-DELTA)*K(T-1) + I(T-1) + (K0*KSTOCK)$TFIRST(T);

M_RK(T).. K(T) * (RK(T)/(R+DELTA))

                =E= K0 * (RK(T)/(R+DELTA))**KVS * PL(T)**(1-KVS) * Y(T);

M_PL(T).. QREF(T) * PL(T)

                =E= (RK(T)/(R+DELTA))**KVS * PL(T)**(1-KVS) * Y(T);

M_PKT..       SUM(TLAST, I(TLAST) - (G+DELTA)*K(TLAST)     )$NLPTERM
            + SUM(TLAST, K(TLAST)*(1-DELTA) + I(TLAST) - TK)$MCPTERM =E= 0;

I_RA..    RA  =E= SUM(T, PL(T)*L0*QREF(T)) + 
                  SUM(TFIRST, PK(TFIRST)*K0*KSTOCK)
                  - (TK * PKT)$MCPTERM;

TERMK$MCPTERM..
         SUM(T$TLAST(T+1),  I(T+1)/I(T) - Y(T+1)/Y(T)) =E= 0;

MODEL MCP
     /PR_Y.Y, PR_U.U, PR_K.K, PR_I.I, 
     M_P.P, M_PU.PU, M_PK.PK, M_RK.RK, M_PL.PL, M_PKT.PKT, 
     I_RA.RA, TERMK.TK/;


Y.L(T)  = QREF(T);
I.L(T)  = (DELTA + G) * K0 * QREF(T);
K.L(T)  = K0 * QREF(T);

P.L(T)  = PREF(T);
RK.L(T) = PREF(T) * (DELTA+R);
PK.L(T) = (1+R) * PREF(T);
PL.L(T) = PREF(T);
PKT.L   = SUM(TLAST, PREF(TLAST));
TK.L    = K0 * (1+G)**CARD(T);

Y.LO(T) = 1.E-5;
I.LO(T) = 1.E-5;


PU.L = PROD(T, (P.L(T)/PREF(T))**ALPHA(T) );

U.L  = ( SUM(T, PL.L(T)*L0*QREF(T)) + SUM(TFIRST, PK.L(TFIRST)*K0*KSTOCK)
        -  TK.L * PKT.L) / PU.L;

RA.FX = U.L * PU.L;

MCP.ITERLIM = 0;
SOLVE MCP USING MCP;

KSTOCK = 0.80;
MCP.ITERLIM = 2000;
SOLVE MCP USING MCP;

PARAMETER       SOLUTION        Solution values;

SOLUTION(T,"PK") = PK.L(T)/P.L(T);
SOLUTION(T,"C") = ALPHA(T) * PU.L * U.L / P.L(T) ;
SOLUTION(T,"I") = I.L(T);
SOLUTION(T,"K") = K.L(T);

DISPLAY SOLUTION;


$TITLE Ramsey Model: Dual NLP Formulation

SET     T /1*50/
SET     TFIRST(T), TLAST(T);
TFIRST(T) = YES$(ORD(T) EQ 1);
TLAST(T)  = YES$(ORD(T) EQ CARD(T)); 

SCALAR          G       Growth rate             /0.02/
                R       Interest rate           /0.05/
                K0      Capital-output ratio    /3.00/
                DELTA   Depreciation rate       /0.07/
                Kstock  Capital index           /1.00/

                KVS     Base year capital value share;
                

PARAMETER       QREF(T) Reference quantity path
                PREF(T) Reference price path;

QREF(T) = (1+G)**(ORD(T)-1);
PREF(T) = (1/(1+R))**(ORD(T)-1);

KVS = K0 * (DELTA + R);

PARAMETER ALPHA;
ALPHA(T) = ((1+G)/(1+R))**(ORD(T)-1);
ALPHA(TLAST) = ALPHA(TLAST) / (1-(1+G)/(1+R));

VARIABLE
     OBJ        Objective price;

POSITIVE VARIABLES
     P(T)       Price of output,
     PC(T)      Consumption price
     PK(T)      Price of capital,
     RK(T)      Price of rental capital,
     PL(T)      Wage rate,
     PT         Price of capital in terminal period;

EQUATIONS 
     OBJDEF     Objective function,
     PCDEF(T)   Defines PC
     PRF_Y(T)   Zero profit condition for output,
     PRF_I(T)   Zero profit condition for investment,
     PRF_K(T)   Zero profit condition for capital;


OBJDEF..        OBJ =E= SUM(T, ALPHA(T)*LOG(PC(T)))
               - SUM(T, PL(T) * QREF(T))
               - SUM(TFIRST, PK(TFIRST)* K0 * KSTOCK);

PCDEF(T)..      PC(T) =E= P(T) / PREF(T);

PRF_Y(T)..     (RK(T)/(R+DELTA))**KVS * (PL(T)/(1-KVS))**(1-KVS) =E= P(T);

PRF_I(T)..     P(T) =E= PK(T+1) + PT$TLAST(T);

PRF_K(T)..     PK(T) + (PT*(G+DELTA))$TLAST(T) =E= RK(T) + PK(T+1)*(1-DELTA);

MODEL DUAL /OBJDEF, PCDEF, PRF_Y, PRF_I, PRF_K /;

P.L(T)    = PREF(T);
PC.L(T)   = 1;
PK.L(T)   = (1+R) * PREF(T);
RK.L(T)   = PREF(T) * (DELTA+R);
PL.L(T)   = PREF(T)*(1-KVS);
PT.L      = SUM(TLAST, PREF(TLAST));

PC.LO(T) = 0.001;
P.LO(T)   = 1E-6;
PK.LO(T)  = 1E-6;
RK.LO(T)  = 1E-6;
PL.LO(T)   = 1E-6;
PT.LO     = 1E-6;

SOLVE DUAL USING NLP MAXIMIZING OBJ;


KSTOCK = 0.80;
SOLVE DUAL USING NLP MAXIMIZING OBJ;

PARAMETER       SOLUTION        Solution values;

SOLUTION(T,"PK") = PK.L(T) / P.L(T);
SOLUTION(T,"C") = -PRF_Y.M(T) + PRF_I.M(T);
SOLUTION(T,"I") = -PRF_I.M(T);
SOLUTION(T,"K") = -PRF_K.M(T);

DISPLAY SOLUTION;


$TITLE  Ramsey Model -- MPSGE formulation with alternative termination methods

$setglobal termcnd MCP

SCALAR NLPTERM /0/, MCPTERM/0/;

%termcnd%term = 1;

SET     T /1*50/;

SET     TFIRST(T)
        TLAST(T);

TFIRST(T) = YES$(ORD(T) EQ 1);
TLAST(T)  = YES$(ORD(T) EQ CARD(T)); 

SCALAR          G       Growth rate             /0.02/
                R       Interest rate           /0.05/
                K0      Capital-output ratio    /3.00/
                DELTA   Depreciation rate       /0.07/
                Kstock  Capital index           /1.00/

                RK0     Base year return to capital
                I0      Base year investment
                C0      Base year consumption
                L0      Base year labor input;
                

I0 = (DELTA + G) * K0;
C0 = 1 - I0;
L0 = 1 - K0 * (DELTA + R);

PARAMETER       QREF(T) Reference quantity path
                PREF(T) Reference price path;

QREF(T) = (1+G)**(ORD(T)-1);
PREF(T) = (1/(1+R))**(ORD(T)-1);

PARAMETER ALPHA;
ALPHA(T) = ((1+G)/(1+R))**(ORD(T)-1);
ALPHA(TLAST) = ALPHA(TLAST) / (1-(1+G)/(1+R));

$ONTEXT

$MODEL:RAMSEY

$SECTORS:
        Y(T)    !       Output
        I(T)    !       Investment
        K(T)    !       Capital stock

$COMMODITIES:
        P(T)    !       Output price
        RK(T)   !       Return to capital
        PK(T)   !       Capital price
        PL(T)   !       Wage rate
        PKT     !       Terminal capital

$CONSUMERS:
        RA      !       Representative agent

$AUXILIARY:
        TK$MCPTERM      !       Post-terminal capital stock

$PROD:Y(T) s:1
        O:P(T)          Q:1
        I:PL(T)         Q:L0
        I:RK(T)         Q:K0    P:(DELTA+R)

$PROD:K(T)$MCPTERM
        O:PK(T+1)       Q:(1-DELTA)
        O:PKT$TLAST(T)  Q:(1-DELTA)
        O:RK(T)         Q:1
        I:PK(T)         Q:1

$PROD:K(T)$NLPTERM
        O:PK(T+1)       Q:(1-DELTA)
        O:RK(T)         Q:1
        I:PK(T)         Q:1
        I:PKT$TLAST(T)  Q:(G+DELTA)


$PROD:I(T)
        O:PK(T+1)       Q:1
        O:PKT$TLAST(T)  Q:1
        I:P(T)          Q:1

$DEMAND:RA  s:1
        D:P(T)$MCPTERM  Q:(QREF(T)*C0)  P:PREF(T)
        D:P(T)$NLPTERM  Q:C0            P:ALPHA(T)
        E:PL(T)         Q:(L0*QREF(T))
        E:PK(TFIRST)    Q:(K0*KSTOCK)
        E:PKT$MCPTERM   Q:-1    R:TK

$REPORT:
        V:C(T)  D:P(T)  DEMAND:RA

$CONSTRAINT:TK$MCPTERM
        SUM(T$TLAST(T+1),  I(T+1)/I(T) - Y(T+1)/Y(T)) =E= 0;

$OFFTEXT
$SYSINCLUDE mpsgeset RAMSEY

Y.L(T)  = QREF(T);
I.L(T)  = I0 * QREF(T);
K.L(T)  = K0 * QREF(T);

P.L(T)  = PREF(T);
RK.L(T) = PREF(T) * (DELTA+R);
PK.L(T) = (1+R) * PREF(T);
PL.L(T) = PREF(T);
PKT.L   = SUM(TLAST, PREF(TLAST));

TK.L    = K0 * (1+G)**CARD(T);

RAMSEY.ITERLIM = 0;
$INCLUDE RAMSEY.GEN
SOLVE RAMSEY USING MCP;

RAMSEY.ITERLIM = 1000;
KSTOCK = 0.80;
$INCLUDE RAMSEY.GEN
SOLVE RAMSEY USING MCP;

PARAMETER       SOLUTION        Solution values;

SOLUTION(T,"PK") = PK.L(T)/P.L(T);
SOLUTION(T,"C") = C.L(T);
SOLUTION(T,"I") = I.L(T);
SOLUTION(T,"K") = K.L(T);

DISPLAY SOLUTION;
