$TITLE Ramsey Model: MCP based on the MPSGE formulation 

$if not setglobal termcnd $setglobal termcnd MCP

SCALAR NLPTERM /0/, MCPTERM/0/;
%termcnd%term = 1;

SET     T /1*50/
SET     TFIRST(T), TLAST(T);
TFIRST(T) = YES$(ORD(T) EQ 1);
TLAST(T)  = YES$(ORD(T) EQ CARD(T)); 

SCALAR          G       Growth rate             /0.02/
                R       Interest rate           /0.05/
                K0      Capital-output ratio    /3.00/
                DELTA   Depreciation rate       /0.07/
                Kstock  Capital index           /1.00/

                KVS     Base year capital value share
                L0      Base year labor input;
                
L0 = 1 - K0 * (DELTA + R);

PARAMETER       QREF(T) Reference quantity path
                PREF(T) Reference price path;

QREF(T) = (1+G)**(ORD(T)-1);
PREF(T) = (1/(1+R))**(ORD(T)-1);

KVS = K0 * (DELTA + R);

PARAMETER ALPHA;
ALPHA(T) = ((1+G)/(1+R))**(ORD(T)-1);
ALPHA(TLAST)$NLPTERM = ALPHA(TLAST) / (1-(1+G)/(1+R));

ALIAS (T,TT);
ALPHA(T) = ALPHA(T) / SUM(TT, ALPHA(TT));

POSITIVE VARIABLES
        Y(T)    Output
        I(T)    Investment
        K(T)    Capital stock
        P(T)    Output price
        RK(T)   Return to capital
        U       Utility
        PU      Unit expenditure function
        PK(T)   Capital price
        PL(T)   Wage rate
        PKT     Terminal capital
        RA      Representative agent
        TK      Post-terminal capital stock;

EQUATIONS
     PR_Y(T)    Zero profit condition for output,
     PR_C(T)    Zero profit condition for consumption,
     PR_K(T)    Zero profit condition for capital,
     PR_I(T)    Zero profit condition for investment,
     PR_U       Zero profit condition for utility,

     M_P(T)     Market clearing for output, 
     M_PK(T)    Market clearing for capital,
     M_RK(T)    Market clearing for rental capital,
     M_PL(T)     Market clearing for labor,
     M_PU       Market clearing for utility,
     M_PKT      Market clearing for terminal investment,
     
     I_RA       Income balance for representative agent,

     TERMK      Terminal constraint for capital stock;

*    Zero profit conditions

PR_Y(T).. (RK(T)/(R+DELTA))**KVS * PL(T)**(1-KVS) =E= P(T);

PR_U..    PROD(T, (P(T)/PREF(T))**ALPHA(T) ) =E= PU;

PR_K(T).. PK(T) + ( (PKT*(G+DELTA))$TLAST(T) )$NLPTERM =E= 
                RK(T) + (1-DELTA)*(PK(T+1) + (PKT$TLAST(T))$MCPTERM);

PR_I(T).. P(T) =E= PK(T+1) + PKT$TLAST(T);

*    Market clearing conditions

M_P(T)..  Y(T) =E= ALPHA(T) * PU * U / P(T) + I(T);

M_PU..    U*PU =E= RA;

M_PK(T).. K(T) =E= (1-DELTA)*K(T-1) + I(T-1) + (K0*KSTOCK)$TFIRST(T);

M_RK(T).. K(T) * (RK(T)/(R+DELTA))

                =E= K0 * (RK(T)/(R+DELTA))**KVS * PL(T)**(1-KVS) * Y(T);

M_PL(T).. QREF(T) * PL(T)

                =E= (RK(T)/(R+DELTA))**KVS * PL(T)**(1-KVS) * Y(T);

M_PKT..       SUM(TLAST, I(TLAST) - (G+DELTA)*K(TLAST)     )$NLPTERM
            + SUM(TLAST, K(TLAST)*(1-DELTA) + I(TLAST) - TK)$MCPTERM =E= 0;

I_RA..    RA  =E= SUM(T, PL(T)*L0*QREF(T)) + 
                  SUM(TFIRST, PK(TFIRST)*K0*KSTOCK)
                  - (TK * PKT)$MCPTERM;

TERMK$MCPTERM..
         SUM(T$TLAST(T+1),  I(T+1)/I(T) - Y(T+1)/Y(T)) =E= 0;

MODEL MCP
     /PR_Y.Y, PR_U.U, PR_K.K, PR_I.I, 
     M_P.P, M_PU.PU, M_PK.PK, M_RK.RK, M_PL.PL, M_PKT.PKT, 
     I_RA.RA, TERMK.TK/;


Y.L(T)  = QREF(T);
I.L(T)  = (DELTA + G) * K0 * QREF(T);
K.L(T)  = K0 * QREF(T);

P.L(T)  = PREF(T);
RK.L(T) = PREF(T) * (DELTA+R);
PK.L(T) = (1+R) * PREF(T);
PL.L(T) = PREF(T);
PKT.L   = SUM(TLAST, PREF(TLAST));
TK.L    = K0 * (1+G)**CARD(T);

PU.L = PROD(T, (P.L(T)/PREF(T))**ALPHA(T) );

U.L  = ( SUM(T, PL.L(T)*L0*QREF(T)) + SUM(TFIRST, PK.L(TFIRST)*K0*KSTOCK)
        -  TK.L * PKT.L) / PU.L;

Y.LO(T) = 1.E-5;
I.LO(T) = 1.E-5;

RA.FX = U.L * PU.L;

MCP.ITERLIM = 0;
SOLVE MCP USING MCP;

KSTOCK = 0.80;
MCP.ITERLIM = 2000;
SOLVE MCP USING MCP;

PARAMETER SOLUTION Solution values;

SOLUTION(T,"PK") = PK.L(T)/P.L(T);
SOLUTION(T,"C") = ALPHA(T) * PU.L * U.L / P.L(T) ;
SOLUTION(T,"I") = I.L(T);
SOLUTION(T,"K") = K.L(T);

DISPLAY SOLUTION;
